/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng;

import java.lang.management.ManagementFactory;
import java.lang.management.RuntimeMXBean;

import com.backup.zipsnap.ng.gui.GUI;
import com.backup.zipsnap.ng.job.Config;
import com.backup.zipsnap.ng.job.Job;

/**
 * Main
 */
public class Main {
	
	/* program version and title */
	private static int versionMajor = 3;
	private static int versionMinor = 1;
	private static int versionPatch = 0;
	private static String versionRelease = "";
	private static String build = "20150518";
	public static String website = "http://user2684.github.io/ZipSnapNG";

	public static double version;
	public static String programTitle;
	public static String lineSeparator = null;
	public static String fileSeparator = null;
	public static String neutralSeparator = "/";
	public static Settings settings = null;

	private static boolean isGUI = false;
	private static String versionString;

	public static void main(String[] args) {
		init();
		/* load the global settings */
		settings = new Settings();
		settings.load();
		checkMemoryUtil();

		/* determine if running in GUI or command line */
		if (args.length > 0 && args[0].equals("gui")) isGUI = true;
		/* init the gui */
		GUI gui = new GUI(isGUI);
		if (isGUI) {
			gui.show();
		} else {
			/* run command line */
			Job job = new Job(isGUI);
			job.run(null,args);
		}

	}
	
	/*
	 * setup version and title
	 */
	private static void init() {
		version = versionMajor+(versionMinor/10)+(versionPatch/100);
		versionString = versionMajor+"."+versionMinor+"."+versionPatch;
		if (!versionRelease.equals("")) versionString = versionString+"-"+versionRelease;
		programTitle = "ZipSnapNG v"+versionString;
		if (!build.equals("")) programTitle = programTitle +" (build "+build+")";
		fileSeparator = System.getProperty("file.separator");
		lineSeparator = System.getProperty("line.separator");
		Config.initialized.set(new Boolean(false));
	}
	
	/*
	 * check if the MemoryUtil class is loaded
	 */
	private static void checkMemoryUtil() {
		if (Main.settings.memoryStats) {
			/* check if the classmexer class is loaded */
			try {
				Class.forName("com.javamex.classmexer.MemoryUtil");
				Class.forName("com.javamex.classmexer.Agent");
			} catch (Exception e) {
				GUI.print(Main.settings.memoryStats+"");
				Main.settings.memoryStats = false;
			}
			/* check if the javaagent has been passed */
			boolean found = false;
			RuntimeMXBean runtimeMxBean = ManagementFactory.getRuntimeMXBean();
			for (String argument : runtimeMxBean.getInputArguments()) {
				if (argument.startsWith("-javaagent:") && argument.contains("classmexer.jar")) found = true;
			}
			if (found == false) Main.settings.memoryStats = false;
		}
	}

}
