/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import com.backup.zipsnap.ng.gui.util.DialogFrame;
import com.backup.zipsnap.ng.gui.util.FileChooser;
import com.backup.zipsnap.ng.gui.util.WideComboBox;
import com.backup.zipsnap.ng.job.Config;
import com.backup.zipsnap.ng.job.unit.FileUnit;

import java.awt.Toolkit;

public class JobConfigFrame extends JFrame {

	private static final long serialVersionUID = 744909374337255630L;
	private Tab tab;
	private GUI gui;
	public static String mergeDirectorySeparator = " => ";
	private JobConfigFrame thisFrame;
	private WideComboBox directoriesToBackup;
	private JButton backupDirectoryAdd;
	private JButton backupDirectoryRemove;
	private WideComboBox backupDirectory;
	private JButton directoriesToBackupAdd;
	private JButton directoriesToBackupRemove;
	private JPasswordField password;
	private JTextField runAfterBackup;
	private JTextField purgeAfterDays;
	@SuppressWarnings("rawtypes")
	private JComboBox restoreMode;
	private WideComboBox restoreInto;
	private JButton restoreIntoAdd;
	private JButton restoreIntoRemove;
	private JTextField filter;


	/*
	 * Create the configuration panel
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public JobConfigFrame(GUI gui, final Tab tab, String title) {
		setIconImage(Toolkit.getDefaultToolkit().getImage(JobConfigFrame.class.getResource("/com/sun/java/swing/plaf/windows/icons/File.gif")));
		this.gui = gui;
		this.tab = tab;
		thisFrame = this;
		setBounds(0, 0, 770, 322);
		setResizable(false);
		getContentPane().setLayout(null);
		setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		setTitle("Job "+title);

		/* draw directoriesToBackup box */
		JLabel directoriesToBackupLabel = new JLabel("Directories to backup: *");
		directoriesToBackupLabel.setBounds(21, 15, 138, 16);
		getContentPane().add(directoriesToBackupLabel);
		directoriesToBackup = new WideComboBox();
		directoriesToBackup.setToolTipText("Location on the filesystem to be included in the backup");
		directoriesToBackup.setBounds(183, 12, 471, 22);
		getContentPane().add(directoriesToBackup);
		directoriesToBackupAdd = new JButton("+");
		directoriesToBackupAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String filename = FileChooser.open(thisFrame, "Select a source directory", FileChooser.action.OPEN,FileChooser.filter.DIRECTORIES);
				if (! filename.equals("")) { 
					/* ensure the item is not already in the list */
					boolean found = false;
					for (int i = 0; i < directoriesToBackup.getItemCount(); i++) {
						if (directoriesToBackup.getItemAt(i).equals(filename)) found = true;
					}
					if (!found) {
						/* add it and allow removing it */
						directoriesToBackup.addItem(filename);
						directoriesToBackupRemove.setEnabled(true);
					}
				}
			}
		});
		directoriesToBackupAdd.setBounds(666, 12, 44, 25);
		directoriesToBackupAdd.setEnabled(true);
		getContentPane().add(directoriesToBackupAdd);
		directoriesToBackupRemove = new JButton("-");
		directoriesToBackupRemove.setEnabled(false);
		directoriesToBackupRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* remove the item and if there are no more items, disable the remove button */
				directoriesToBackup.removeItemAt(directoriesToBackup.getSelectedIndex());
				if (directoriesToBackup.getItemCount() == 0) directoriesToBackupRemove.setEnabled(false);
			}
		});
		directoriesToBackupRemove.setBounds(714, 12, 44, 25);
		getContentPane().add(directoriesToBackupRemove);

		/* draw archiveDirectory box */
		JLabel archiveDirectoryLabel = new JLabel("Backup Directory: *");
		archiveDirectoryLabel.setBounds(45, 46, 126, 16);
		getContentPane().add(archiveDirectoryLabel);
		backupDirectory = new WideComboBox();
		backupDirectory.setToolTipText("Location on the filesystem where the backup will be stored");
		backupDirectory.setBounds(183, 43, 471, 22);
		getContentPane().add(backupDirectory);
		backupDirectoryAdd = new JButton("+");
		backupDirectoryAdd.setEnabled(true);
		backupDirectoryAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String filename = FileChooser.open(thisFrame, "Select a destination directory", FileChooser.action.OPEN,FileChooser.filter.DIRECTORIES);
				if (! filename.equals("")) {
					/* add the item, enable removal, disable addition */
					backupDirectory.addItem(filename);
					backupDirectoryAdd.setEnabled(false);
					backupDirectoryRemove.setEnabled(true);
				}
			}
		});
		backupDirectoryAdd.setBounds(666, 42, 44, 25);
		getContentPane().add(backupDirectoryAdd);
		backupDirectoryRemove = new JButton("-");
		backupDirectoryRemove.setEnabled(false);
		backupDirectoryRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* remove the item, enable the addition, disable removal */
				backupDirectory.removeItemAt(backupDirectory.getSelectedIndex());
				backupDirectoryAdd.setEnabled(true);
				backupDirectoryRemove.setEnabled(false);
			}
		});
		backupDirectoryRemove.setBounds(714, 42, 44, 25);
		getContentPane().add(backupDirectoryRemove);

		/* draw password box */
		JLabel lblPassword = new JLabel("Password:");
		lblPassword.setHorizontalAlignment(SwingConstants.RIGHT);
		lblPassword.setBounds(71, 81, 88, 16);
		getContentPane().add(lblPassword);
		password = new JPasswordField();
		password.setColumns(10);
		password.setToolTipText("Password for securing the archive (default: empty)");
		password.setBounds(183, 78, 144, 22);
		getContentPane().add(password);

		/* draw runAfterBackup box */
		JLabel lblRunAfterBackup = new JLabel("Run After Backup:");
		lblRunAfterBackup.setHorizontalAlignment(SwingConstants.RIGHT);
		lblRunAfterBackup.setBounds(12, 116, 150, 16);
		getContentPane().add(lblRunAfterBackup);
		runAfterBackup = new JTextField();
		runAfterBackup.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				String filename = FileChooser.open(thisFrame, "Select a script to execute", FileChooser.action.OPEN,FileChooser.filter.NONE);
				if (! filename.equals("")) {
					/* add the item */
					runAfterBackup.setText(filename);
				}
			}
		});

		runAfterBackup.setToolTipText("Command to run after the backup. The backup directory will passed as first parameter, the backup file as second parameter (default: empty)");
		runAfterBackup.setColumns(10);
		runAfterBackup.setBounds(183, 113, 569, 22);
		getContentPane().add(runAfterBackup);

		/* draw purgeAfterDays box */
		JLabel lblPurgeAfter = new JLabel("Purge after:");
		lblPurgeAfter.setHorizontalAlignment(SwingConstants.RIGHT);
		lblPurgeAfter.setBounds(398, 81, 97, 16);
		getContentPane().add(lblPurgeAfter);
		purgeAfterDays = new JTextField();
		purgeAfterDays.setToolTipText("Number of days after which snapshots older than the threshold will be deleted (default: empty)");
		purgeAfterDays.setBounds(499, 78, 102, 22);
		getContentPane().add(purgeAfterDays);
		JLabel lblDays = new JLabel("days");
		lblDays.setBounds(611, 81, 56, 16);
		getContentPane().add(lblDays);

		/* restore mode box */
		JLabel lblRestoreInto = new JLabel("Restore Into:");
		lblRestoreInto.setHorizontalAlignment(SwingConstants.RIGHT);
		lblRestoreInto.setBounds(52, 150, 107, 16);
		getContentPane().add(lblRestoreInto);
		restoreMode = new JComboBox();
		restoreMode.setToolTipText("Instruct how to restore the snapshot");
		restoreMode.addItem("the original location");
		restoreMode.addItem("a different directory");
		restoreMode.addItem("a different set of directories");
		restoreMode.setBounds(183, 147, 546, 22);
		getContentPane().add(restoreMode);
		restoreInto = new WideComboBox();
		restoreInto.setVisible(false);
		restoreInto.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				/* enable the buttons based on the selected item */
				if (restoreMode.getSelectedIndex() == 1) {
					if (restoreInto.getItemCount() > 0) {
						restoreIntoAdd.setEnabled(false);
						restoreIntoRemove.setEnabled(true);						
					} else {
						restoreIntoAdd.setEnabled(true);
						restoreIntoRemove.setEnabled(false);						
					}

				}
				if (restoreMode.getSelectedIndex() == 2 && restoreInto.getItemCount() > 0) {
					if (((String) restoreInto.getSelectedItem()).endsWith(mergeDirectorySeparator)) {
						restoreIntoAdd.setEnabled(true);
						restoreIntoRemove.setEnabled(false);
					} else {
						restoreIntoRemove.setEnabled(true);
						restoreIntoAdd.setEnabled(false);
					}
				}
			}
		});
		restoreMode.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				restoreModeLoader(true);
			}
		});
		restoreInto.setBounds(183, 182, 471, 22);
		getContentPane().add(restoreInto);
		restoreIntoAdd = new JButton("+");
		restoreIntoAdd.setVisible(false);
		restoreIntoAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (restoreMode.getSelectedIndex() == 1) { // extract into
					String filename = FileChooser.open(thisFrame,"Select a directory to extract the backup into", FileChooser.action.OPEN,FileChooser.filter.DIRECTORIES);
					if (! filename.equals("")) {
						/* add the item, disable addition, enable removal */
						restoreInto.addItem(filename);
						restoreInto.setSelectedItem(filename);
						restoreIntoAdd.setEnabled(false);
						restoreIntoRemove.setEnabled(true);
					}
				}
				if (restoreMode.getSelectedIndex() == 2) { // merge directory
					String[] mergeItem = ((String)restoreInto.getSelectedItem()).split(mergeDirectorySeparator);
					String filename = FileChooser.open(thisFrame,"Select where to merge "+mergeItem[0], FileChooser.action.OPEN,FileChooser.filter.DIRECTORIES);
					if (! filename.equals("")) {
						/* remove the item, add a new one with the new value, disable addition, enable removal */
						restoreInto.removeItem(restoreInto.getSelectedItem());
						String item = mergeItem[0] + mergeDirectorySeparator+filename;
						restoreInto.addItem(item);
						restoreInto.setSelectedItem(item);
						restoreIntoAdd.setEnabled(false);
						restoreIntoRemove.setEnabled(true);
					}
				}
			}
		});
		restoreIntoAdd.setEnabled(false);
		restoreIntoAdd.setBounds(666, 181, 44, 25);
		getContentPane().add(restoreIntoAdd);
		restoreIntoRemove = new JButton("-");
		restoreIntoRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (restoreMode.getSelectedIndex() == 1) {
					/* remove the item, enable addition, disable removal */
					restoreInto.removeItemAt(directoriesToBackup.getSelectedIndex());
					restoreIntoAdd.setEnabled(true);
					restoreIntoRemove.setEnabled(false);
				}
				if (restoreMode.getSelectedIndex() == 2) {
					/* remove the item, add a new one without the right value, enable addition, disable removal */
					String[] mergeItem = ((String)restoreInto.getSelectedItem()).split(mergeDirectorySeparator);
					restoreInto.removeItem(restoreInto.getSelectedItem());
					String item = mergeItem[0] + mergeDirectorySeparator;
					restoreInto.addItem(item);
					restoreInto.setSelectedItem(item);
					restoreIntoAdd.setEnabled(true);
					restoreIntoRemove.setEnabled(false);
				}
			}
		});
		restoreIntoRemove.setEnabled(false);
		restoreIntoRemove.setVisible(false);
		restoreIntoRemove.setBounds(714, 182, 44, 25);
		getContentPane().add(restoreIntoRemove);

		/* draw filter box */
		JLabel lblFilter = new JLabel("Filter:");
		lblFilter.setHorizontalAlignment(SwingConstants.RIGHT);
		lblFilter.setBounds(103, 222, 56, 16);
		getContentPane().add(lblFilter);
		filter = new JTextField();
		filter.setToolTipText("Regular Expression filter to be applied for all the commands (default: empty)");
		filter.setColumns(10);
		filter.setBounds(183, 219, 569, 22);
		getContentPane().add(filter);

		/* draw the control buttons */
		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				thisFrame.setVisible(false);
			}
		});
		btnCancel.setBounds(654, 254, 97, 25);
		getContentPane().add(btnCancel);

		JButton btnSave = new JButton("Save");
		btnSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveConfig();
			}
		});
		btnSave.setIcon(new ImageIcon(JobConfigFrame.class.getResource("/javax/swing/plaf/metal/icons/ocean/floppy.gif")));
		btnSave.setBounds(545, 254, 97, 25);
		getContentPane().add(btnSave);
		
		JLabel lblMandatorySetting = new JLabel("*: mandatory field");
		lblMandatorySetting.setBounds(21, 258, 328, 16);
		getContentPane().add(lblMandatorySetting);
		setVisible(false);
	}
	
	/*
	 * handle restore into based on restore mode value
	 */
	@SuppressWarnings("unchecked")
	private void restoreModeLoader(boolean changed) {
		restoreInto.removeAllItems();
		restoreIntoAdd.setEnabled(false);
		restoreIntoRemove.setEnabled(false);
		if ((changed && restoreMode.getSelectedIndex() == 0) || (!changed && tab.config.restoreMode == Config.restoreType.origin)) {
			/* just hide the restore into box */
			restoreMode.setSelectedIndex(0);
			restoreInto.setVisible(false);
			restoreIntoAdd.setVisible(false);
			restoreIntoRemove.setVisible(false);
		}
		if ((changed && restoreMode.getSelectedIndex() == 1) || (!changed && tab.config.restoreMode == Config.restoreType.extractInto)) {
			/* show the restore into box */
			restoreMode.setSelectedIndex(1);
			restoreInto.setToolTipText("The files will be extracted here with the origin location converted into a sub-folder (e.g. c:\\dir\\subdir\\file.txt -> c:\\restoreInto\\c-dir-subdir\\file.txt)");
			restoreInto.setVisible(true);
			restoreIntoAdd.setVisible(true);
			restoreIntoRemove.setVisible(true);
			/* load the item */
			if (! changed) restoreInto.addItem(tab.config.setting2string("restoreInto"));
			if (restoreInto.getItemCount() > 0) restoreIntoRemove.setEnabled(true);
			else restoreIntoAdd.setEnabled(true);
		}
		if ((changed && restoreMode.getSelectedIndex() == 2) || (!changed && tab.config.restoreMode == Config.restoreType.mergeDirectory)) {
			restoreMode.setSelectedIndex(2);
			restoreInto.setToolTipText("The files will be restored by merging the original and destination directories configured in the mapping");
			/* show the restore into box */
			restoreInto.setVisible(true);
			restoreIntoAdd.setVisible(true);
			restoreIntoRemove.setVisible(true);
			restoreInto.removeAllItems();
			if (changed) {
				/* add the directories in the archive */
				for (FileUnit u : tab.config.directories) restoreInto.addItem(u.nativeName+" "+mergeDirectorySeparator);
			} else {
				
				/* load the pairs from the configuration */
				List<String> pairs = new ArrayList<String>();
				if (tab.config.setting2string("restoreInto").contains(";")) pairs = Arrays.asList(tab.config.setting2string("restoreInto").split(";"));
				else pairs.add(tab.config.setting2string("restoreInto"));
				for (String pair : pairs) {
					final String[] match = pair.split("->");
					String source = match[0];
					String destination = match[1];
					if (!source.equals("") && !destination.equals("")) {
						restoreInto.addItem(source+mergeDirectorySeparator+destination);
					}
				}
			}
		}
	}

	/*
	 * populate the panel with the configuration
	 */
	@SuppressWarnings("unchecked")
	public void loadConfig() {
		/* load directoriesToBackup */
		directoriesToBackup.removeAllItems();
		for (String directory : tab.config.setting2string("directoriesToBackup").split("\\;")) directoriesToBackup.addItem(directory);
		if (directoriesToBackup.getItemCount() > 0) directoriesToBackupRemove.setEnabled(true);
		directoriesToBackupAdd.setEnabled(true);

		/* load backupDirectory */
		backupDirectory.removeAllItems();
		if (! tab.config.setting2string("backupDirectory").equals("")) backupDirectory.addItem(tab.config.setting2string("backupDirectory"));
		if (backupDirectory.getItemCount() > 0) {
			backupDirectoryAdd.setEnabled(false);
			backupDirectoryRemove.setEnabled(true);
		}
		else {
			backupDirectoryAdd.setEnabled(true);
			backupDirectoryRemove.setEnabled(false);
		}

		/* load other settings */
		password.setText(tab.config.setting2string("password"));
		runAfterBackup.setText(tab.config.setting2string("runAfterBackup"));
		purgeAfterDays.setText(tab.config.setting2string("purgeAfterDays"));
		filter.setText(tab.config.setting2string("filter"));

		/* load restoreInto */
		restoreModeLoader(false);
	}

	/*
	 * save the configuration
	 */
	public void saveConfig() {
		ArrayList<String> errors = new ArrayList<String>();
		/* check for errors */
		if (directoriesToBackup.getItemCount() == 0) errors.add("There should be at least one directory to backup");
		if (backupDirectory.getItemCount() == 0) errors.add("The archive directory is missing");
		if (! purgeAfterDays.getText().equals("") && ! purgeAfterDays.getText().matches("[0-9]+")) errors.add("Purge after days must be a number");
		if (restoreMode.getSelectedIndex() == 1) {
			if (restoreInto.getItemCount() == 0) errors.add("The restore directory is missing");
		}
		if (restoreMode.getSelectedIndex() == 2) {
			for (int i = 0; i < restoreInto.getItemCount(); i++) {
				if (((String) restoreInto.getItemAt(i)).endsWith(mergeDirectorySeparator)) {
					errors.add("At least one restore directory is missing");
					break;
				}
			}
		}
		try {
			@SuppressWarnings("unused")
			Pattern filterPattern = Pattern.compile(filter.getText());
		} catch (Exception e) {
			errors.add("Invalid filter pattern provided");
		}

		if (errors.size() == 0) {
			/* save the config */
			thisFrame.setVisible(false);
			tab.actionUpdateJob();
		}
		else {
			/* show an error message */
			String errorString = "";
			for (String s : errors) errorString = errorString + s+"\n";
			gui.windowsManager.show(new DialogFrame(DialogFrame.type.ERROR, errorString));
		}
	}

	/*
	 * apply the settings to the config
	 */
	public void applyConfig()  {
		String directoriesToBackupString = "";
		for (int i = 0; i < directoriesToBackup.getItemCount(); i++) {
			directoriesToBackupString = directoriesToBackupString + directoriesToBackup.getItemAt(i);
			if (i != directoriesToBackup.getItemCount()-1) directoriesToBackupString = directoriesToBackupString + ";";
		}
		tab.config.string2setting("directoriesToBackup",directoriesToBackupString);
		tab.config.string2setting("backupDirectory",(String) backupDirectory.getSelectedItem());
		tab.config.string2setting("password",new String(password.getPassword()));
		tab.config.string2setting("runAfterBackup",runAfterBackup.getText());
		tab.config.string2setting("purgeAfterDays",purgeAfterDays.getText());
		tab.config.string2setting("filter",filter.getText());
		String restoreIntoString = "";
		for (int i = 0; i < restoreInto.getItemCount(); i++) {
			restoreIntoString = restoreIntoString + restoreInto.getItemAt(i);
			if (i != restoreInto.getItemCount()-1) restoreIntoString = restoreIntoString + ";";
		}
		restoreIntoString = restoreIntoString.replaceAll("\\s*"+mergeDirectorySeparator+"\\s*", "->");
		tab.config.string2setting("restoreInto",restoreIntoString);
	}
}
