/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import com.backup.zipsnap.ng.gui.util.DialogFrame;
import com.backup.zipsnap.ng.gui.util.FileTree;
import com.backup.zipsnap.ng.gui.util.FileTreeFrame;
import com.backup.zipsnap.ng.gui.util.WideComboBox;
import com.backup.zipsnap.ng.job.Config;

public class JobPanel extends JPanel {
	private static final long serialVersionUID = 1610962327883679241L;
	private GUI gui;
	private Tab tab;
	private File configFile;
	private WideComboBox backupDirectory;
	private WideComboBox directoriesToBackup;
	private WideComboBox restoreInto;
	private JTable snapshotsTable;
	private TaskPanel jobTaskPanel;
	private JButton btnBackup;
	private JButton btnRestore;
	private JLabel archiveInfo;
	private FileTreeFrame snapshotFileFrame;
	private WideComboBox filter;
	public static enum status { RUNNING,READY; }


	/*
	 * Create the job panel
	 */
	public JobPanel(final GUI gui, final Tab tab, final File configFile) {
		this.configFile = configFile;
		this.gui = gui;
		this.tab = tab;
		/* set the layout */
		setLayout(null);
		setBounds(0, 0, 1014, 660);
		
		/* init the snapshot info frame */
		snapshotFileFrame = new FileTreeFrame(gui);
		
		/* draw archiveDirectory box */
		JLabel archiveDirectoryLabel = new JLabel("Archive directory:");
		archiveDirectoryLabel.setBounds(57, 46, 126, 16);
		add(archiveDirectoryLabel);
		backupDirectory = new WideComboBox();
		backupDirectory.setBounds(183, 43, 778, 22);
		add(backupDirectory);
		
		/* draw directoriesToBackup box */
		JLabel directoriesToBackupLabel = new JLabel("Directories to backup:");
		directoriesToBackupLabel.setBounds(33, 16, 126, 16);
		add(directoriesToBackupLabel);
		directoriesToBackup = new WideComboBox();
		directoriesToBackup.setBounds(183, 12, 778, 25);
		add(directoriesToBackup);
		
		/* draw RestoreInto box */
		JLabel restoreIntoLabel = new JLabel("Restore Into:");
		restoreIntoLabel.setBounds(85, 74, 98, 16);
		add(restoreIntoLabel);
		restoreInto = new WideComboBox();
		restoreInto.setBounds(183, 71, 576, 22);
		add(restoreInto);
		
		/* draw the filter box*/
		JLabel lblFilter = new JLabel("Filter:");
		lblFilter.setBounds(771, 74, 43, 16);
		add(lblFilter);
		filter = new WideComboBox();
		filter.setBounds(819, 71, 142, 22);
		add(filter);
		
		/* draw snapshot table */
		JScrollPane snapshotsScrollPane = new JScrollPane();
		snapshotsScrollPane.setBounds(33, 107, 930, 133);
		add(snapshotsScrollPane);
		snapshotsTable = new JTable();
		snapshotsTable.setFillsViewportHeight(true);
		snapshotsTable.setModel(new DefaultTableModel(
			new Object[][] {
			},
			new String[] {
				"#", "Computer", "Encrypted", "Deleted", "Date", "Snapshot Files (compressed)", "Total Files (uncompressed)"
			}
		) {
			private static final long serialVersionUID = -3104538783887101743L;
			@SuppressWarnings("rawtypes")
			Class[] columnTypes = new Class[] {
				String.class, String.class, Boolean.class, Boolean.class, String.class, String.class, String.class
			};
			public Class<?> getColumnClass(int columnIndex) {
				return columnTypes[columnIndex];
			}
			boolean[] columnEditables = new boolean[] {
					false, false, false, false, false, false, false
			};
			public boolean isCellEditable(int row, int column) {
				return columnEditables[column];
			}
		});
		snapshotsTable.getColumnModel().getColumn(0).setMinWidth(50);
		snapshotsTable.getColumnModel().getColumn(0).setPreferredWidth(50);
		snapshotsTable.getColumnModel().getColumn(1).setPreferredWidth(140);
		snapshotsTable.getColumnModel().getColumn(1).setMinWidth(140);
		snapshotsTable.getColumnModel().getColumn(2).setPreferredWidth(50);
		snapshotsTable.getColumnModel().getColumn(2).setMinWidth(50);
		snapshotsTable.getColumnModel().getColumn(3).setPreferredWidth(50);
		snapshotsTable.getColumnModel().getColumn(3).setMinWidth(50);
		snapshotsTable.getColumnModel().getColumn(4).setMinWidth(200);
		snapshotsTable.getColumnModel().getColumn(4).setPreferredWidth(200);
		snapshotsTable.getColumnModel().getColumn(5).setPreferredWidth(220);
		snapshotsTable.getColumnModel().getColumn(5).setMinWidth(220);
		snapshotsTable.getColumnModel().getColumn(6).setPreferredWidth(220);
		snapshotsTable.getColumnModel().getColumn(6).setMinWidth(220);
		DefaultTableCellRenderer centerRenderer = new DefaultTableCellRenderer();
		centerRenderer.setHorizontalAlignment( JLabel.CENTER );
		snapshotsTable.setDefaultRenderer(String.class, centerRenderer);
		snapshotsScrollPane.setViewportView(snapshotsTable);
		archiveInfo = new JLabel("");
		archiveInfo.setHorizontalAlignment(SwingConstants.CENTER);
		archiveInfo.setBounds(334, 246, 321, 21);
		add(archiveInfo);
		
		/* draw backup button */
		btnBackup = new JButton("Backup");
		btnBackup.setIcon(new ImageIcon(JobPanel.class.getResource("/javax/swing/plaf/metal/icons/ocean/iconify-pressed.gif")));
		btnBackup.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				actionBackup();
			}
		});
		btnBackup.setBounds(179, 242, 130, 25);
		add(btnBackup);
		
		/* draw restore button */
		btnRestore = new JButton("Restore");
		btnRestore.setIcon(new ImageIcon(JobPanel.class.getResource("/javax/swing/plaf/metal/icons/ocean/maximize-pressed.gif")));
		btnRestore.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				actionRestore();
			}
		});
		btnRestore.setBounds(692, 242, 130, 25);
		add(btnRestore);
	}
	
	/*
	 * run backup action
	 */
	public void actionBackup() {
		if (jobTaskPanel != null) remove(jobTaskPanel);
		jobTaskPanel = new TaskPanel(gui,TaskPanel.type.BACKUP);
		add(jobTaskPanel);
		jobTaskPanel.repaint();
		String cmd[] = {configFile.getAbsolutePath(),"backup"};
		tab.runJob(cmd);
	}
	
	/*
	 * run restore action
	 */
	public void actionRestore() {
		int selectedRow = snapshotsTable.getSelectedRow();
		final String selectedSnapshot = selectedRow == -1 ? ""+snapshotsTable.getValueAt(0, 0) : ""+snapshotsTable.getValueAt(selectedRow, 0);
		DialogFrame question = (DialogFrame) gui.windowsManager.show(new DialogFrame(DialogFrame.type.YES_NO,"Do you want to proceed with restoring snapshot #"+selectedSnapshot+"?"));
		question.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent action) {
				if (action.getNewValue() == DialogFrame.action.YES) {
					if (jobTaskPanel != null) remove(jobTaskPanel);
					jobTaskPanel = new TaskPanel(gui,TaskPanel.type.RESTORE);
					add(jobTaskPanel);
					jobTaskPanel.repaint();
					String cmd[] = {configFile.getAbsolutePath(),"restore",selectedSnapshot};
					tab.runJob(cmd);
				}
			}
		});
	}
	
	/*
	 * run purge action
	 */
	public void actionPurge() {
		if (jobTaskPanel != null) remove(jobTaskPanel);
		jobTaskPanel = new TaskPanel(gui,TaskPanel.type.PURGE);
		add(jobTaskPanel);
		jobTaskPanel.repaint();
		String cmd[] = {configFile.getAbsolutePath(),"purge"};
		tab.runJob(cmd);
	}
	
	/*
	 * run snapshot info action
	 */
	public void actionListFiles() {
		int selectedRow = snapshotsTable.getSelectedRow();
		final String selectedSnapshot = selectedRow == -1 ? ""+snapshotsTable.getValueAt(0, 0) : ""+snapshotsTable.getValueAt(selectedRow, 0);
		//snapshotFileFrame.init();
		snapshotFileFrame.fileTree.setStatus(FileTree.status.RUNNING);
		gui.windowsManager.show(snapshotFileFrame);
		String cmd[] = {configFile.getAbsolutePath(),"infoSnapshot",selectedSnapshot};
		tab.runJob(cmd);
	}
	
	/*
	 * stop the job
	 */
	public void actionStop() {
		tab.jobEngine.stop();
		gui.windowsManager.closeAll();
		gui.windowsManager.show(new DialogFrame(DialogFrame.type.WARNING,"The job has been interrupted"));
	}

	/*
	 * set the status to the object
	 */
	public void setStatus(status what) {
		if (what == status.RUNNING) {
			btnBackup.setEnabled(false);
			btnRestore.setEnabled(false);
			tab.statusBar.reset();
		}
		else if (what == status.READY) {
			btnBackup.setEnabled(true);
			btnRestore.setEnabled(true);
		}
	}
	
	
	/*
	 *  add a new snapshot to the table
	 */
	public void setSnapshot(Object[] obj) {
		((DefaultTableModel)snapshotsTable.getModel()).insertRow(0, obj);
	}
	
	/*
	 * make the job slider progressing
	 */
	public void setJobProgress(String obj) {
		jobTaskPanel.jobProgress.setValue(jobTaskPanel.jobProgress.getValue()+1);
		jobTaskPanel.taskProgress.setVisible(false);
		jobTaskPanel.taskEta.setVisible(false);
		jobTaskPanel.jobProgress.repaint();
	}
	
	/*
	 * set the task progress bar to the given value
	 */
	public void setTaskProgress(int value) {
		jobTaskPanel.taskProgress.setVisible(true);
		jobTaskPanel.taskProgress.setValue(value);
	}
	
	/*
	 * set the task eta to the given value
	 */
	public void setTaskEta(String eta) {
		jobTaskPanel.taskEta.setVisible(true);
		jobTaskPanel.taskEta.setText(eta);
	}
	
	/*
	 * add a file to the file tree of the task panel
	 */
	public void setTaskFile(String[] obj) {
		jobTaskPanel.fileTree.add((String[])obj);
		jobTaskPanel.fileTreeFrame.fileTree.add((String[])obj);
	}
	
	/*
	 * add a file to the file tree of the snapshot info
	 */
	public void setSnapshotFile(String[] obj) {
		snapshotFileFrame.fileTree.add((String[])obj);
	}
	
	/*
	 * set the snapshot info filetree as ready
	 */
	public void setSnapshotFileReady() {
		snapshotFileFrame.fileTree.setStatus(FileTree.status.READY);
	}
	
	/*
	 * set no new items
	 */
	public void setNoFile() {
		jobTaskPanel.fileTree.setStatus(FileTree.status.EMPTY);
		jobTaskPanel.fileTreeFrame.fileTree.setStatus(FileTree.status.EMPTY);
	}
	
	/*
	 * the summary is ready to show
	 */
	public void setSummaryReady() {
		jobTaskPanel.fileTree.setStatus(FileTree.status.READY);
		jobTaskPanel.fileTreeFrame.fileTree.setStatus(FileTree.status.READY);
	}

	/*
	 * show the task panel label
	 */
	public void setTaskPanelLabel(String obj) {
		jobTaskPanel.taskPanelLabel.setText(obj);
	}

	/*
	 * info on the archive size
	 */
	public void setArchiveInfo(String obj) {
		archiveInfo.setText(obj);
	}
	
	
	/*
	 * populate the panel based on the config
	 */
	@SuppressWarnings("unchecked")
	public void loadConfig(Config config) {
		/* handle backupDirectory */
		backupDirectory.removeAllItems();
		backupDirectory.addItem(config.setting2string("backupDirectory"));
		/* handle directoriesToBackup */
		directoriesToBackup.removeAllItems();
		for (String directory : tab.config.setting2string("directoriesToBackup").split("\\;")) {
			directoriesToBackup.addItem(directory);
		}
		/* handle restoreInto */
		restoreInto.removeAllItems();
		if (config.restoreMode == Config.restoreType.origin) {
			restoreInto.addItem("the original location");
		} 
		else if (config.restoreMode == Config.restoreType.extractInto) {
			restoreInto.addItem(tab.config.setting2string("restoreInto"));
		}
		else if (config.restoreMode == Config.restoreType.mergeDirectory) {
			String[] pairs = tab.config.setting2string("restoreInto").split("\\;");
			for (String pair : pairs) {
				final String[] match = pair.split("->");
				String source = match[0];
				String destination = match[1];
				if (!source.equals("") && !destination.equals("")) {
					restoreInto.addItem(source+" "+JobConfigFrame.mergeDirectorySeparator+" "+destination);
				}
			}
		}
		if (restoreInto.getItemCount() > 0) restoreInto.setSelectedIndex(0);
		if (config.filter != null) filter.addItem(config.filter.pattern());
	}
	
	/*
	 * reset the panel
	 */
	public void reset() {
		/* empty the snapshot table */
		clearSnapshotTable();
		backupDirectory.removeAllItems();
		directoriesToBackup.removeAllItems();
		restoreInto.removeAllItems();
		filter.removeAllItems();
	}
	
	/*
	 * clear the snapshot table
	 */
	public void clearSnapshotTable() {
		DefaultTableModel dm = (DefaultTableModel) snapshotsTable.getModel();
		int rowCount = dm.getRowCount();
		for (int i = rowCount - 1; i >= 0; i--) dm.removeRow(i);
	}
	
	/*
	 * handle overwrite/delete input
	 */
	public void setConfirmOverwriteDelete(DialogFrame question) {
		question.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent action) {
				if (action.getNewValue() == DialogFrame.action.YES) {
					try { tab.jobEngine.inwriter.write("Y\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
				else if (action.getNewValue() == DialogFrame.action.NO) {
					try { tab.jobEngine.inwriter.write("N\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
				else if (action.getNewValue() == DialogFrame.action.ALWAYS) {
					try { tab.jobEngine.inwriter.write("A\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
				else if (action.getNewValue() == DialogFrame.action.NEVER) {
					try { tab.jobEngine.inwriter.write("N\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
			}
		});
	}
	
	/*
	 * ask the user confirmation before proceeding with the job
	 */
	public void setConfirmSummary(DialogFrame question) {
		question.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent action) {
				if (action.getNewValue() == DialogFrame.action.YES) {
					try { tab.jobEngine.inwriter.write("Y\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
				else if (action.getNewValue() == DialogFrame.action.NO) {
					try { tab.jobEngine.inwriter.write("N\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
			}
		});
	}
	
	/*
	 * ask the user to confirm the warning
	 */
	public void setConfirmWarning(DialogFrame question) {
		question.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent action) {
				if (action.getNewValue() == DialogFrame.action.OK) {
					try { tab.jobEngine.inwriter.write("O\n".getBytes());} 
					catch (IOException e) {e.printStackTrace(); }
				}
			}
		});
	}
}
