/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui;

import java.awt.Font;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;

import com.backup.zipsnap.ng.gui.util.FileTree;
import com.backup.zipsnap.ng.gui.util.FileTreeFrame;

public class TaskPanel extends JPanel {

	private static final long serialVersionUID = -6315797711407685194L;
	private type thisType;
	private JLabel lblTitle;
	public JSlider jobProgress;
	public JProgressBar taskProgress;
	public FileTree fileTree;
	public FileTreeFrame fileTreeFrame;
	public JLabel taskEta;
	public JLabel taskPanelLabel;
	public static enum type { BACKUP,RESTORE,PURGE; }

	/*
	 * Create the task panel
	 */
	public TaskPanel(GUI gui, type type) {
		this.thisType = type;
		setBorder(new BevelBorder(BevelBorder.RAISED, null, null, null, null));
		setBounds(33, 272, 928, 382);

		/* draw the title */
		lblTitle = new JLabel(getTitle());
		lblTitle.setHorizontalAlignment(SwingConstants.CENTER);
		lblTitle.setFont(new Font("Tahoma", Font.BOLD, 18));
		lblTitle.setBounds(355, 13, 197, 31);
		add(lblTitle);

		/* draw the job progress slider */
		ArrayList<String> sliderLabels = getSliderLabels();
		jobProgress = new JSlider();
		jobProgress.setBounds(12, 50, 904, 81);
		jobProgress.setPaintTicks(true);
		jobProgress.setSnapToTicks(true);
		jobProgress.setPaintLabels(true);
		jobProgress.setValue(0);
		jobProgress.setMaximum(sliderLabels.size()-1);
		jobProgress.setMajorTickSpacing(1);
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		for (int i = 0; i < sliderLabels.size(); i++) {
			labelTable.put(new Integer(i), new JLabel(sliderLabels.get(i)) );
		}
		jobProgress.setLabelTable( labelTable );
		setLayout(null);
		add(jobProgress);

		/* draw the file tree */
		fileTree = new FileTree(gui);
		fileTree.setBounds(51, 164, 851, 187);
		fileTreeFrame = new FileTreeFrame(gui);
		fileTree.fileTreeFrame = fileTreeFrame;
		add(fileTree);
		
		/* draw the labels */
		taskPanelLabel = new JLabel("");
		taskPanelLabel.setBounds(51, 353, 851, 24);
		add(taskPanelLabel);
		
		/* draw the progress bar */
		taskProgress = new JProgressBar();
		taskProgress.setBounds(341, 132, 250, 19);
		taskProgress.setStringPainted(true);
		taskProgress.setVisible(false);
		add(taskProgress);
		taskEta = new JLabel("");
		taskEta.setBounds(611, 132, 155, 16);
		taskEta.setVisible(false);
		add(taskEta);
	}

	/*
	 * return the labels for the task slider
	 */
	private ArrayList<String> getSliderLabels() {
		ArrayList<String> sliderLabels = new ArrayList<String>();
		if (thisType == type.BACKUP) {
			sliderLabels.add("<html>Loading<br>Job</html>");
			sliderLabels.add("<html>Initializing</html>");
			sliderLabels.add("<html>Assessing<br>Source<br>Directories</html>");
			sliderLabels.add("<html>Calculating<br>Differential<br>Backup</html>");
			sliderLabels.add("<html>Backup<br>Summary</html>");
			sliderLabels.add("<html>Backup<br>in Progress</html>");
			sliderLabels.add("<html>Finalizing</html>");
			sliderLabels.add("<html>Run Custom<br>Command</html>");
			sliderLabels.add("<html>Purge<br>Archives</html>");
			sliderLabels.add("<html>Done</html>");
		}
		else if (thisType == type.RESTORE) {
			sliderLabels.add("<html>Loading<br>Job</html>");
			sliderLabels.add("<html>Initializing</html>");
			sliderLabels.add("<html>Assessing<br>Destination<br>Directories</html>");
			sliderLabels.add("<html>Calculating<br>Differential<br>Restore</html>");
			sliderLabels.add("<html>Restore<br>Summary</html>");
			sliderLabels.add("<html>Restore<br>in Progress</html>");
			sliderLabels.add("<html>Finalizing</html>");
			sliderLabels.add("<html>Verifying<br>Restore</html>");
			sliderLabels.add("<html>Done</html>");
		}
		else if (thisType == type.PURGE) {
			sliderLabels.add("<html>Loading<br>Job</html>");
			sliderLabels.add("<html>Initializing</html>");
			sliderLabels.add("<html>Purge<br>Summary</html>");
			sliderLabels.add("<html>Purge<br>in Progress</html>");
			sliderLabels.add("<html>Finalizing</html>");
			sliderLabels.add("<html>Done</html>");
		}
		return sliderLabels;
	}
	
	/*
	 * return the title for this panel 
	 */
	private String getTitle() {
		if (thisType == type.BACKUP) return "Backup";
		if (thisType == type.RESTORE) return "Restore";
		if (thisType == type.PURGE) return "Purge";
		return "";
	}
}
