/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.gui.util;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.regex.Pattern;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.gui.GUI;


public class FileTree extends JPanel implements Cloneable {
	private static final long serialVersionUID = -1358737371001808688L;
	private DefaultMutableTreeNode rootNode;
	private DefaultTreeModel treeModel;
	private JTree tree;
	private String loadingLabel = "Loading...";
	private String emptyLabel = "No items need to be saved/restored/purged";
	public FileTreeFrame fileTreeFrame = null;
	public static enum status { RUNNING,READY,EMPTY; }
	private int count = 0;

	/*
	 * create a file tree 
	 */
	public FileTree(final GUI gui) {
		super(new GridLayout(1, 0));
		/* init the tree model*/
		rootNode = new DefaultMutableTreeNode(loadingLabel);
		treeModel = new DefaultTreeModel(rootNode);
		/* init the tree */
		tree = new JTree(treeModel);
		tree.setEditable(false);
		tree.setShowsRootHandles(true);
		tree.setEnabled(false);
		/* add the tree to the panel */
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		JScrollPane scrollPane = new JScrollPane(tree);
		add(scrollPane);

		/* add right click menu */
		MouseAdapter ma = new MouseAdapter() {
			private void myPopupEvent(MouseEvent e) {
				int x = e.getX();
				int y = e.getY();
				final JTree tree = (JTree)e.getSource();
				final TreePath path = tree.getPathForLocation(x, y);
				if (path == null) return;
				tree.setSelectionPath(path);
				JPopupMenu popup = new JPopupMenu();
				final JMenuItem refreshMenuItem = new JMenuItem("Expand All");
				refreshMenuItem.addActionListener(new ActionListener(){
					public void actionPerformed(ActionEvent actionEvent) {
						expandAll(path);
					}
				});
				popup.add(refreshMenuItem);
				if (fileTreeFrame != null) { 
					popup.add(new JSeparator());
					final JMenuItem maximizeMenuItem = new JMenuItem("Open in a new window");
					maximizeMenuItem.addActionListener(new ActionListener(){
						public void actionPerformed(ActionEvent actionEvent) {
							gui.windowsManager.show(fileTreeFrame);
						}
					});
					popup.add(maximizeMenuItem);
				}
				popup.show(tree, x, y);
			}
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger()) myPopupEvent(e);
			}
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) myPopupEvent(e);
			}
		};
		tree.addMouseListener(ma);
	}

	/*
	 * prepare an object (root, filename, attributes) to be added to the tree
	 */
	public void add(String[] obj) {
		ArrayList<String> items = new ArrayList<String>();
		count++;
		if (count >= Main.settings.fileTreeMaxItems) {
			if (count == Main.settings.fileTreeMaxItems) {
				items.add("...the number of items exceeds the configured "+Main.settings.fileTreeMaxItems+", the output will be suppressed...");
				add((DefaultMutableTreeNode)treeModel.getRoot(),items,0,"");	
			}
		} else {
			String root = obj[0];
			String file = obj[1];
			String attributes = obj[2];
			/* split the path in the single directories */
			items.add(root);
			String[] dirs = file.split(Pattern.quote(File.separator));
			String prevItem = "";
			for (int i = 0; i < dirs.length; i++) {
				if (i == 0) items.add(dirs[i]);
				else items.add(prevItem+File.separator+dirs[i]);
				prevItem = items.get(items.size()-1); 
			}
			add((DefaultMutableTreeNode)treeModel.getRoot(),items,0,attributes);
		}
	}

	/*
	 * add the object to the tree
	 */
	private void add(DefaultMutableTreeNode parentNode, ArrayList<String> items,int start,String attributes) {
		/* gone beyond the last item */
		if (start >= items.size()) return;
		/* if there are no children a new node has to be created */
		if (parentNode.getChildCount() == 0) addNewNode(parentNode,items,start,attributes);
		else {
			boolean found = false;
			/* search for a children with the same name */
			for (int i = 0; i < parentNode.getChildCount(); i++) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) parentNode.getChildAt(i);
				/* found a children with the same name */
				if (((String)node.getUserObject()).equals(getNodeName(items,start,attributes))) {
					found = true;
					/* go on along the path */
					add(node,items,start+1,attributes);
					break;
				} 
			}
			/* no children found, a new node has to be created */
			if (!found)  addNewNode(parentNode,items,start,attributes);
		}
	}

	/*
	 * add a new node to the parentNode
	 */
	private void addNewNode(DefaultMutableTreeNode parentNode, ArrayList<String> items,int start,String attributes) {
		DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(getNodeName(items,start,attributes));
		treeModel.insertNodeInto(newNode, parentNode, parentNode.getChildCount());
		add(newNode,items,start+1,attributes);
		repaint();
	}

	/*
	 * generate the name to be assigned to a given node
	 */
	private String getNodeName(ArrayList<String> items,int start,String attributes) {
		if (start == items.size()-1) return items.get(start)+attributes;
		else return items.get(start);
	}

	/* 
	 * expand all the nodes along a give path
	 * */
	private void expandAll(TreePath parent) {
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0) {
			for (@SuppressWarnings("unchecked")
			Enumeration<TreeNode> e = node.children(); e.hasMoreElements();) {
				TreeNode treeNode = (TreeNode) e.nextElement();
				TreePath path = parent.pathByAddingChild(treeNode);
				tree.expandPath(parent);
				expandAll(path);
			}
		}
	}
	
	/*
	 * set the status to the FileTree
	 */
	public void setStatus(status what) {
		if (what == status.RUNNING) {
			count = 0;
			rootNode = new DefaultMutableTreeNode(loadingLabel);
			treeModel = new DefaultTreeModel(rootNode);
			tree.setModel(treeModel);
			tree.setEnabled(false);
			tree.setRootVisible(true);
			treeModel.reload();
		}
		else if (what == status.READY) {
			if (((DefaultMutableTreeNode)treeModel.getRoot()).getUserObject().equals(emptyLabel)) return;
			tree.expandPath(new TreePath(rootNode.getPath()));
			tree.setEnabled(true);
			tree.setRootVisible(false);
			treeModel.reload();
		}
		else if (what == status.EMPTY) {
			count = 0;
			((DefaultMutableTreeNode)treeModel.getRoot()).setUserObject(emptyLabel);
			tree.setRootVisible(true);
			tree.setEnabled(false);
			treeModel.reload();
		}
	}
}