/**
 * TERMS AND CONDITIONS:
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.backup.zipsnap.ng.job.util;

import java.util.ArrayList;
import java.io.File;
import java.io.PrintWriter;
import java.io.PrintStream;

import com.backup.zipsnap.ng.Main;
import com.backup.zipsnap.ng.job.util.FileIO;
import com.backup.zipsnap.ng.job.util.UserIO;

/**
 * Handle logging to file
 */
public class Logger
{
  private final ArrayList<String> buffer;
  private PrintWriter pw = null;
  
  public Logger() {
    buffer = new ArrayList<String>();
  }
  
  /**
   * Open a log file
   */
  public void openLog(File logFile) {
    final String logFileName = logFile.getName();
    if (logFileName.contains(".0.")) return;
    UserIO.print(UserIO.out.DEBUG,"Creating new log file " + logFileName);
    FileIO.checkFile(logFile);
    try {
      pw = new PrintWriter(new PrintStream(logFile));
    } catch (Exception e) {
      UserIO.reportError("Unable to create log file \"" + logFileName + "\": " +UserIO.getExceptionMessage(e));
    }
    /* if something has been buffered, time to flush it */
    if (buffer.size() > 0) {
      for (String s : buffer) {
        pw.print(s);
      }
      buffer.clear();
      pw.flush();
    }
  }
  
  /**
   * Log an entry
   */
  public void log(String entry) {
    if (pw == null) {
      /* the log has not been opened yet, buffer the entry*/
      buffer.add(entry);
    } else {
      /* log the entry */
      pw.print(entry);
      if (Main.settings.logFlushAlways) pw.flush();
    }
  }
  
  /**
   * Close a log file
   */
  public void closeLog() {
    UserIO.print(UserIO.out.DEBUG,"Closing the log file");
    if (pw != null) {
      pw.flush();
      pw.close();
    }
  }
}